@@ -1,4 +1,5 @@
 import asyncio
+import sys
 import threading
 from asyncio import QueueEmpty as AsyncQueueEmpty
 from asyncio import QueueFull as AsyncQueueFull
@@ -8,12 +9,12 @@ from queue import Empty as SyncQueueEmpty
 from queue import Full as SyncQueueFull
 from typing import Any, Callable, Deque, Generic, List, Optional, Set, TypeVar
 
-__version__ = "0.6.0"
+__version__ = "0.6.2"
 __all__ = ("Queue", "PriorityQueue", "LifoQueue")
 
 
 T = TypeVar("T")
-OptInt = Optional[int]
+OptFloat = Optional[float]
 
 
 current_loop = getattr(asyncio, "get_running_loop", None)
@@ -36,6 +37,9 @@ class Queue(Generic[T]):
         self._all_tasks_done = threading.Condition(self._sync_mutex)
 
         self._async_mutex = asyncio.Lock()
+        if sys.version_info[:3] == (3, 10, 0):
+            # Workaround for Python 3.10 bug, see #358:
+            getattr(self._async_mutex, '_get_loop', lambda: None)()
         self._async_not_empty = asyncio.Condition(self._async_mutex)
         self._async_not_full = asyncio.Condition(self._async_mutex)
         self._finished = asyncio.Event()
@@ -69,6 +73,8 @@ class Queue(Generic[T]):
             self._closing = True
             for fut in self._pending:
                 fut.cancel()
+            self._finished.set()  # unblocks all async_q.join()
+            self._all_tasks_done.notify_all()  # unblocks all sync_q.join()
 
     async def wait_closed(self) -> None:
         # should be called from loop after close().
@@ -172,7 +178,7 @@ class Queue(Generic[T]):
 
     def _check_closing(self) -> None:
         if self._closing:
-            raise RuntimeError("Modification of closed queue is forbidden")
+            raise RuntimeError("Operation on the closed queue is forbidden")
 
 
 class _SyncQueueProxy(Generic[T]):
@@ -225,9 +231,11 @@ class _SyncQueueProxy(Generic[T]):
 
         When the count of unfinished tasks drops to zero, join() unblocks.
         """
+        self._parent._check_closing()
         with self._parent._all_tasks_done:
             while self._parent._unfinished_tasks:
                 self._parent._all_tasks_done.wait()
+                self._parent._check_closing()
 
     def qsize(self) -> int:
         """Return the approximate size of the queue (not reliable!)."""
@@ -261,7 +269,7 @@ class _SyncQueueProxy(Generic[T]):
         """
         return 0 < self._parent._maxsize <= self._parent._qsize()
 
-    def put(self, item: T, block: bool = True, timeout: OptInt = None) -> None:
+    def put(self, item: T, block: bool = True, timeout: OptFloat = None) -> None:
         """Put an item into the queue.
 
         If optional args 'block' is true and 'timeout' is None (the default),
@@ -295,7 +303,7 @@ class _SyncQueueProxy(Generic[T]):
             self._parent._sync_not_empty.notify()
             self._parent._notify_async_not_empty(threadsafe=True)
 
-    def get(self, block: bool = True, timeout: OptInt = None) -> T:
+    def get(self, block: bool = True, timeout: OptFloat = None) -> T:
         """Remove and return an item from the queue.
 
         If optional args 'block' is true and 'timeout' is None (the default),
@@ -513,6 +521,7 @@ class _AsyncQueueProxy(Generic[T]):
         """
         while True:
             with self._parent._sync_mutex:
+                self._parent._check_closing()
                 if self._parent._unfinished_tasks == 0:
                     break
             await self._parent._finished.wait()
